@@ -6,6 +6,7 @@ lav_model_estimate <- function(lavmodel       = NULL,
                                lavdata        = NULL,
                                lavoptions     = NULL,
                                lavcache       = list(),
+                               start          = "model",
                                do.fit         = TRUE) {
 
     estimator     <- lavoptions$estimator
@@ -19,8 +20,12 @@ lav_model_estimate <- function(lavmodel       = NULL,
         group.weight <- TRUE
     }
 
-    # temp test
-    if(lavoptions$partrace) {
+    # backwards compatibility < 0.6-11
+    if(is.null(lavoptions$optim.partrace)) {
+        lavoptions$optim.partrace <- FALSE
+    }
+
+    if(lavoptions$optim.partrace) {
         # fx + parameter values
         PENV <- new.env()
         PENV$PARTRACE <- matrix(NA, nrow=0, ncol=lavmodel@nx.free + 1L)
@@ -29,6 +34,32 @@ lav_model_estimate <- function(lavmodel       = NULL,
     # starting values (ignoring equality constraints)
     x.unpack <- lav_model_get_parameters(lavmodel)
 
+    # override? use simple instead? (new in 0.6-7)
+    if(start == "simple") {
+        START <- numeric(length(lavpartable$lhs))
+        # set loadings to 0.7
+        loadings.idx <- which(lavpartable$free > 0L &
+                              lavpartable$op == "=~")
+        if(length(loadings.idx) > 0L) {
+            START[loadings.idx] <- 0.7
+        }
+        # set (only) variances to 1
+        var.idx <- which(lavpartable$free > 0L &
+                         lavpartable$op == "~~" &
+                         lavpartable$lhs == lavpartable$rhs)
+        if(length(var.idx) > 0L) {
+            START[var.idx] <- 1
+        }
+
+        if(lavmodel@ceq.simple.only) {
+            x.unpack <- START[ lavpartable$free > 0L &
+                               !duplicated(lavpartable$free) ]
+        } else {
+            x.unpack <- START[ lavpartable$free > 0L ]
+        }
+    }
+
+
     # 1. parameter scaling (to handle data scaling, not parameter scaling)
     parscale <- rep(1.0, length(x.unpack))
 
@@ -76,12 +107,30 @@ lav_model_estimate <- function(lavmodel       = NULL,
                             cov.x = lavsamplestats@cov.x)
         } else {
             # needs good estimates for lv variances!
+            # if there is a single 'marker' indicator, we could use
+            # its observed variance as an upper bound
+
+            # for the moment, set them to 1.0 (instead of 0.05)
+
+            # TODO: USE Bentler's 1982 approach to get an estimate of
+            # VETA; use those diagonal elements...
+            # but only if we have 'marker' indicators for each LV
+            LV.VAR <- vector("list", lavmodel@ngroups)
+            for(g in seq_len(lavmodel@ngroups)) {
+                mm.in.group <- 1:lavmodel@nmat[g] + cumsum(c(0,lavmodel@nmat))[g]
+                MLIST     <- lavmodel@GLIST[ mm.in.group ]
+                LAMBDA <- MLIST$lambda
+                n.lv <- ncol(LAMBDA)
+                LV.VAR[[g]] <- rep(1.0, n.lv)
+            }
+
             parscale <- lav_standardize_all(lavobject = NULL,
                             est = rep(1, length(lavpartable$lhs)),
                             #est.std = rep(1, length(lavpartable$lhs)),
                             # here, we use whatever the starting values are
                             # for the latent variances...
                             cov.std = FALSE, ov.var = OV.VAR,
+                            lv.var = LV.VAR,
                             lavmodel = lavmodel, lavpartable = lavpartable,
                             cov.x = lavsamplestats@cov.x)
         }
@@ -92,7 +141,13 @@ lav_model_estimate <- function(lavmodel       = NULL,
         if(length(var.idx) > 0L) {
             parscale[var.idx] <- sqrt(abs(parscale[var.idx]))
         }
-        parscale <- parscale[ lavpartable$free > 0 ]
+
+        if(lavmodel@ceq.simple.only) {
+            parscale <- parscale[ lavpartable$free > 0 &
+                                  !duplicated(lavpartable$free) ]
+        } else {
+            parscale <- parscale[ lavpartable$free > 0 ]
+        }
     }
     # parscale should obey the equality constraints
     if(lavmodel@eq.constraints && lavoptions$optim.parscale != "none") {
@@ -134,50 +189,57 @@ lav_model_estimate <- function(lavmodel       = NULL,
         cat("start.x = ", start.x, "\n")
     }
 
-
-    # bounds? (new in 0.6-2)
+    # user-specified bounds? (new in 0.6-2)
     if(is.null(lavpartable$lower)) {
         lower <- -Inf
     } else {
-        lower <- lavpartable$lower[ lavpartable$free > 0 ]
-        if(lavmodel@eq.constraints) {
-            # collect non -Inf elements
-            not.inf.val <- lower[is.finite(lower)]
-
-            lowerb <- lower
-            lowerb[lower == -Inf] <- -999999
-
-            # pack
-            l.pack <- as.numeric( (lowerb - lavmodel@eq.constraints.k0) %*%
-                                  lavmodel@eq.constraints.K )
-            lower <- l.pack
-            lower[!(lower %in% not.inf.val)] <- -Inf
+        if(lavmodel@ceq.simple.only) {
+            free.idx <- which(lavpartable$free > 0L &
+                              !duplicated(lavpartable$free))
+            lower <- lavpartable$lower[free.idx]
+        } else if(lavmodel@eq.constraints) {
+            # bounds have no effect any longer....
+            warning("lavaan warning: bounds have no effect in the presence of linear equality constraints")
+            lower <- -Inf
+        } else {
+            lower <- lavpartable$lower[ lavpartable$free > 0L ]
         }
     }
     if(is.null(lavpartable$upper)) {
         upper <- +Inf
     } else {
-        upper <- lavpartable$upper[ lavpartable$free > 0 ]
-        if(lavmodel@eq.constraints) {
-            # collect non -Inf elements
-            not.inf.val <- upper[is.finite(upper)]
-
-            upperb <- upper
-            upperb[upper == Inf] <- 999999
-
-            # pack
-            l.pack <- as.numeric( (upperb - lavmodel@eq.constraints.k0) %*%
-                                  lavmodel@eq.constraints.K )
-            upper <- l.pack
-            upper[!(upper %in% not.inf.val)] <- Inf
+        if(lavmodel@ceq.simple.only) {
+            free.idx <- which(lavpartable$free > 0L &
+                              !duplicated(lavpartable$free))
+            upper <- lavpartable$upper[free.idx]
+        } else if(lavmodel@eq.constraints) {
+            # bounds have no effect any longer....
+            if(is.null(lavpartable$lower)) {
+                # bounds have no effect any longer....
+                warning("lavaan warning: bounds have no effect in the presence of linear equality constraints")
+            }
+            upper <- +Inf
+        } else {
+            upper <- lavpartable$upper[ lavpartable$free > 0L ]
         }
     }
 
-
-
+    # check for inconsistent lower/upper bounds
+    # this may happen if we have equality constraints; qr() may switch
+    # the sign...
+    bad.idx <- which(lower > upper)
+    if(length(bad.idx) > 0L) {
+        # switch
+        #tmp <- lower[bad.idx]
+        #lower[bad.idx] <- upper[bad.idx]
+        #upper[bad.idx] <- tmp
+        lower[bad.idx] <- -Inf
+        upper[bad.idx] <- +Inf
+    }
 
     # function to be minimized
-    objective_function <- function(x, verbose = FALSE, infToMax = FALSE) {
+    objective_function <- function(x, verbose = FALSE, infToMax = FALSE,
+                                   debug = FALSE) {
 
         # 3. standard deviations to variances
         # WARNING: x is still packed here!
@@ -206,8 +268,7 @@ lav_model_estimate <- function(lavmodel       = NULL,
                                   lavsamplestats = lavsamplestats,
                                   lavdata        = lavdata,
                                   lavcache       = lavcache,
-                                  verbose        = verbose,
-                                  forcePD        = FALSE)
+                                  verbose        = verbose)
 
         # only for PML: divide by N (to speed up convergence)
         if(estimator == "PML") {
@@ -217,7 +278,8 @@ lav_model_estimate <- function(lavmodel       = NULL,
 
 
         if(debug || verbose) {
-            cat("Objective function  = ", sprintf("%18.16f", fx), "\n", sep="")
+            cat("  objective function  = ",
+                sprintf("%18.16f", fx), "\n", sep="")
         }
         if(debug) {
             #cat("Current unconstrained parameter values =\n")
@@ -226,20 +288,23 @@ lav_model_estimate <- function(lavmodel       = NULL,
             cat("Current free parameter values =\n"); print(x); cat("\n")
         }
 
-        if(lavoptions$partrace) {
+        if(lavoptions$optim.partrace) {
             PENV$PARTRACE <- rbind(PENV$PARTRACE, c(fx, x))
         }
 
         # for L-BFGS-B
         #if(infToMax && is.infinite(fx)) fx <- 1e20
         if(!is.finite(fx)) {
+            fx.group <- attr(fx, "fx.group")
             fx <- 1e20
+            attr(fx, "fx.group") <- fx.group # only for lav_model_fit()
         }
 
         fx
     }
 
-    gradient_function <- function(x, verbose = FALSE, infToMax = FALSE) {
+    gradient_function <- function(x, verbose = FALSE, infToMax = FALSE,
+                                  debug = FALSE) {
 
         # transform variances back
         #if(lavoptions$optim.var.transform == "sqrt" &&
@@ -270,7 +335,7 @@ lav_model_estimate <- function(lavmodel       = NULL,
                                  type           = "free",
                                  group.weight   = group.weight, ### check me!!
                                  verbose        = verbose,
-                                 forcePD        = TRUE)
+                                 ceq.simple     = lavmodel@ceq.simple.only)
 
         if(debug) {
             cat("Gradient function (analytical) =\n"); print(dx); cat("\n")
@@ -307,7 +372,7 @@ lav_model_estimate <- function(lavmodel       = NULL,
         dx
     }
 
-    gradient_function_numerical <- function(x, verbose=FALSE) {
+    gradient_function_numerical <- function(x, verbose=FALSE, debug = FALSE) {
 
         # NOTE: no need to 'tranform' anything here (var/eq)
         # this is done anyway in objective_function
@@ -322,10 +387,10 @@ lav_model_estimate <- function(lavmodel       = NULL,
             x.left <- x.left2 <- x.right <- x.right2 <- x
             x.left[i]  <- x[i] - h; x.left2[i]  <- x[i] - 2*h
             x.right[i] <- x[i] + h; x.right2[i] <- x[i] + 2*h
-            fx.left   <- objective_function(x.left)
-            fx.left2  <- objective_function(x.left2)
-            fx.right  <- objective_function(x.right)
-            fx.right2 <- objective_function(x.right2)
+            fx.left   <- objective_function(x.left,   verbose = FALSE, debug = FALSE)
+            fx.left2  <- objective_function(x.left2,  verbose = FALSE, debug = FALSE)
+            fx.right  <- objective_function(x.right,  verbose = FALSE, debug = FALSE)
+            fx.right2 <- objective_function(x.right2, verbose = FALSE, debug = FALSE)
             dx[i] <- (fx.left2 - 8*fx.left + 8*fx.right - fx.right2)/(12*h)
         }
 
@@ -339,10 +404,22 @@ lav_model_estimate <- function(lavmodel       = NULL,
         dx
     }
 
+    gradient_function_numerical_complex <- function(x, verbose=FALSE, debug = FALSE) {
+
+        dx <- Re(lav_func_gradient_complex(func = objective_function, x = x,
+                                        h = sqrt(.Machine$double.eps)))
+        # does not work if pnorm is involved... (eg PML)
+
+        if(debug) {
+            cat("Gradient function (numerical complex) =\n"); print(dx); cat("\n")
+        }
+
+        dx
+    }
+
 
     # check if the initial values produce a positive definite Sigma
     # to begin with -- but only for estimator="ML"
-    #if(estimator %in% c("ML","PML","FML","MML")) {
     if(estimator %in% c("ML","FML","MML")) {
         Sigma.hat <- computeSigmaHat(lavmodel, extra=TRUE, debug=lavoptions$debug)
         for(g in 1:ngroups) {
@@ -350,6 +427,7 @@ lav_model_estimate <- function(lavmodel       = NULL,
                 group.txt <- ifelse(ngroups > 1,
                                     paste(" in group ",g,".",sep=""), ".")
                 if(debug) print(Sigma.hat[[g]])
+                print(Sigma.hat[[g]][,])
                 stop("lavaan ERROR: initial model-implied matrix (Sigma) is not positive definite;\n  check your model and/or starting parameters", group.txt)
                 x <- start.x
                 fx <- as.numeric(NA)
@@ -380,6 +458,15 @@ lav_model_estimate <- function(lavmodel       = NULL,
     }
 
 
+    # first try: check if starting values return a finite value
+    fx <- objective_function(start.x, verbose = verbose, debug = debug)
+    if(!is.finite(fx)) {
+        # emergency change of start.x
+        start.x <- start.x / 10
+    }
+
+
+
     # first some nelder mead steps? (default = FALSE)
     INIT_NELDER_MEAD <- lavoptions$optim.init_nelder_mead
 
@@ -389,6 +476,8 @@ lav_model_estimate <- function(lavmodel       = NULL,
             GRADIENT <- gradient_function
         } else if(lavoptions$optim.gradient %in% c("numerical", "numeric")) {
             GRADIENT <- gradient_function_numerical
+        } else if(lavoptions$optim.gradient %in% c("numeric.complex", "complex")){
+            GRADIENT <- gradient_function_numerical_complex
         } else if(lavoptions$optim.gradient %in% c("NULL", "null")) {
             GRADIENT <- NULL
         } else {
@@ -422,11 +511,19 @@ lav_model_estimate <- function(lavmodel       = NULL,
             }
         }
     } else {
-        OPTIMIZER <- "NLMINB.CONSTR"
+        if(is.null(lavoptions$optim.method)) {
+            OPTIMIZER <- "NLMINB.CONSTR"
+        } else {
+            OPTIMIZER <- toupper(lavoptions$optim.method)
+            stopifnot(OPTIMIZER %in% c("NLMINB.CONSTR", "NLMINB", "NONE"))
+        }
+        if(OPTIMIZER == "NLMINB") {
+            OPTIMIZER <- "NLMINB.CONSTR"
+        }
     }
 
     if(INIT_NELDER_MEAD) {
-        if(verbose) cat("Initial Nelder-Mead step:\n")
+        if(verbose) cat("  initial Nelder-Mead step:\n")
         trace <- 0L; if(verbose) trace <- 1L
         optim.out <- optim(par=start.x,
                            fn=objective_function,
@@ -435,7 +532,7 @@ lav_model_estimate <- function(lavmodel       = NULL,
                            #             parscale=SCALE,
                            #             trace=trace),
                            hessian=FALSE,
-                           verbose=verbose)
+                           verbose=verbose, debug=debug)
         cat("\n")
         start.x <- optim.out$par
     }
@@ -443,7 +540,7 @@ lav_model_estimate <- function(lavmodel       = NULL,
 
 
     if(OPTIMIZER == "NLMINB0") {
-        if(verbose) cat("Quasi-Newton steps using NLMINB0 (no analytic gradient):\n")
+        if(verbose) cat("  quasi-Newton steps using NLMINB0 (no analytic gradient):\n")
         #if(debug) control$trace <- 1L;
         control.nlminb <- list(eval.max=20000L,
                                iter.max=10000L,
@@ -468,25 +565,37 @@ lav_model_estimate <- function(lavmodel       = NULL,
                             upper=upper,
                             control=control,
                             scale=SCALE,
-                            verbose=verbose)
+                            verbose=verbose, debug=debug)
         if(verbose) {
-            cat("convergence status (0=ok): ", optim.out$convergence, "\n")
-            cat("nlminb message says: ", optim.out$message, "\n")
-            cat("number of iterations: ", optim.out$iterations, "\n")
-            cat("number of function evaluations [objective, gradient]: ",
+            cat("  convergence status (0=ok): ", optim.out$convergence, "\n")
+            cat("  nlminb message says: ", optim.out$message, "\n")
+            cat("  number of iterations: ", optim.out$iterations, "\n")
+            cat("  number of function evaluations [objective, gradient]: ",
                 optim.out$evaluations, "\n")
         }
 
+        # try again
+        if(optim.out$convergence != 0L) {
+             optim.out <- nlminb(start=start.x,
+                            objective=objective_function,
+                            gradient=NULL,
+                            lower=lower,
+                            upper=upper,
+                            control=control,
+                            scale=SCALE,
+                            verbose=verbose, debug=debug)
+        }
+
         iterations <- optim.out$iterations
         x          <- optim.out$par
-        if(optim.out$convergence == 0) {
+        if(optim.out$convergence == 0L) {
             converged <- TRUE
         } else {
             converged <- FALSE
         }
 
     } else if(OPTIMIZER == "NLMINB") {
-        if(verbose) cat("Quasi-Newton steps using NLMINB:\n")
+        if(verbose) cat("  quasi-Newton steps using NLMINB:\n")
         #if(debug) control$trace <- 1L;
         control.nlminb <- list(eval.max=20000L,
                                iter.max=10000L,
@@ -511,22 +620,23 @@ lav_model_estimate <- function(lavmodel       = NULL,
                             upper=upper,
                             control=control,
                             scale=SCALE,
-                            verbose=verbose)
+                            verbose=verbose, debug=debug)
         if(verbose) {
-            cat("convergence status (0=ok): ", optim.out$convergence, "\n")
-            cat("nlminb message says: ", optim.out$message, "\n")
-            cat("number of iterations: ", optim.out$iterations, "\n")
-            cat("number of function evaluations [objective, gradient]: ",
+            cat("  convergence status (0=ok): ", optim.out$convergence, "\n")
+            cat("  nlminb message says: ", optim.out$message, "\n")
+            cat("  number of iterations: ", optim.out$iterations, "\n")
+            cat("  number of function evaluations [objective, gradient]: ",
                 optim.out$evaluations, "\n")
         }
 
         iterations <- optim.out$iterations
         x          <- optim.out$par
-        if(optim.out$convergence == 0) {
+        if(optim.out$convergence == 0L) {
             converged <- TRUE
         } else {
             converged <- FALSE
         }
+
     } else if(OPTIMIZER == "BFGS") {
 
         # warning: Bollen example with estimator=GLS does NOT converge!
@@ -550,12 +660,12 @@ lav_model_estimate <- function(lavmodel       = NULL,
                            method="BFGS",
                            control=control,
                            hessian=FALSE,
-                           verbose=verbose)
+                           verbose=verbose, debug=debug)
         if(verbose) {
-            cat("convergence status (0=ok): ", optim.out$convergence, "\n")
-            cat("optim BFGS message says: ", optim.out$message, "\n")
+            cat("  convergence status (0=ok): ", optim.out$convergence, "\n")
+            cat("  optim BFGS message says: ", optim.out$message, "\n")
             #cat("number of iterations: ", optim.out$iterations, "\n")
-            cat("number of function evaluations [objective, gradient]: ",
+            cat("  number of function evaluations [objective, gradient]: ",
                 optim.out$counts, "\n")
         }
 
@@ -591,13 +701,13 @@ lav_model_estimate <- function(lavmodel       = NULL,
                            upper=upper,
                            control=control,
                            hessian=FALSE,
-                           verbose=verbose,
+                           verbose=verbose, debug=debug,
                            infToMax=TRUE)
         if(verbose) {
-            cat("convergence status (0=ok): ", optim.out$convergence, "\n")
-            cat("optim L-BFGS-B message says: ", optim.out$message, "\n")
+            cat("  convergence status (0=ok): ", optim.out$convergence, "\n")
+            cat("  optim L-BFGS-B message says: ", optim.out$message, "\n")
             #cat("number of iterations: ", optim.out$iterations, "\n")
-            cat("number of function evaluations [objective, gradient]: ",
+            cat("  number of function evaluations [objective, gradient]: ",
                 optim.out$counts, "\n")
         }
 
@@ -635,17 +745,19 @@ lav_model_estimate <- function(lavmodel       = NULL,
                                    gradient=GRADIENT,
                                    control=control,
                                    scale=SCALE,
-                                   verbose=verbose,
+                                   verbose=verbose, debug=debug,
+                                   lower=lower,
+                                   upper=upper,
                                    cin = cin, cin.jac = cin.jac,
                                    ceq = ceq, ceq.jac = ceq.jac,
                                    control.outer = ocontrol
                                   )
         if(verbose) {
-            cat("convergence status (0=ok): ", optim.out$convergence, "\n")
-            cat("nlminb.constr message says: ", optim.out$message, "\n")
-            cat("number of outer iterations: ", optim.out$outer.iterations, "\n")
-            cat("number of inner iterations: ", optim.out$iterations, "\n")
-            cat("number of function evaluations [objective, gradient]: ",
+            cat("  convergence status (0=ok): ", optim.out$convergence, "\n")
+            cat("  nlminb.constr message says: ", optim.out$message, "\n")
+            cat("  number of outer iterations: ", optim.out$outer.iterations, "\n")
+            cat("  number of inner iterations: ", optim.out$iterations, "\n")
+            cat("  number of function evaluations [objective, gradient]: ",
                 optim.out$evaluations, "\n")
         }
 
@@ -661,11 +773,108 @@ lav_model_estimate <- function(lavmodel       = NULL,
         iterations <- 0L
         converged <- TRUE
         control <- list()
-        optim.out <- list()
+
+        # if inequality constraints, add con.jac/lambda
+        # needed for df!
+        if(length(lavmodel@ceq.nonlinear.idx) == 0L &&
+           length(lavmodel@cin.linear.idx)    == 0L &&
+           length(lavmodel@cin.nonlinear.idx) == 0L) {
+            optim.out <- list()
+        } else {
+            # if inequality constraints, add con.jac/lambda
+            # needed for df!
+
+            optim.out <- list()
+            if(is.null(body(lavmodel@ceq.function))) {
+                 ceq <- function(x, ...) { return( numeric(0) ) }
+            } else {
+                 ceq <- lavmodel@ceq.function
+            }
+            if(is.null(body(lavmodel@cin.function))) {
+                 cin <- function(x, ...) { return( numeric(0) ) }
+            } else {
+                 cin <- lavmodel@cin.function
+            }
+            ceq0 <- ceq(start.x)
+            cin0 <- cin(start.x)
+            con0 <- c(ceq0, cin0)
+            JAC <- rbind(numDeriv::jacobian(ceq, x = start.x),
+                         numDeriv::jacobian(cin, x = start.x))
+            nceq <- length( ceq(start.x) )
+            ncin <- length( cin(start.x) )
+            ncon <- nceq + ncin
+            ceq.idx <- cin.idx <- integer(0)
+            if(nceq > 0L) ceq.idx <- 1:nceq
+            if(ncin > 0L) cin.idx <- nceq + 1:ncin
+            cin.flag <- rep(FALSE, length(ncon))
+            if(ncin > 0L) cin.flag[cin.idx] <- TRUE
+
+            inactive.idx <- integer(0L)
+            cin.idx <- which(cin.flag)
+            if(ncin > 0L) {
+                slack <- 1e-05
+                inactive.idx <- which(cin.flag & con0 > slack)
+            }
+            attr(JAC, "inactive.idx") <- inactive.idx
+            attr(JAC, "cin.idx") <- cin.idx
+            attr(JAC, "ceq.idx") <- ceq.idx
+
+            optim.out$con.jac <- JAC
+            optim.out$lambda <- rep(0, ncon)
+        }
     }
 
     fx <- objective_function(x) # to get "fx.group" attribute
 
+    # check convergence
+    warn.txt <- ""
+    if(converged) {
+        # check.gradient
+        if(!is.null(GRADIENT) &&
+           OPTIMIZER %in% c("NLMINB", "BFGS", "L-BFGS-B")) {
+
+            # compute unscaled gradient
+            dx <- GRADIENT(x)
+
+            # NOTE: unscaled gradient!!!
+            if(converged && lavoptions$check.gradient &&
+               any(abs(dx) > lavoptions$optim.dx.tol)) {
+
+                # ok, identify the non-zero elements
+                non.zero <- which(abs(dx) > lavoptions$optim.dx.tol)
+
+                # which ones are 'boundary' points, defined by lower/upper?
+                bound.idx <- integer(0L)
+                if(!is.null(lavpartable$lower)) {
+                    bound.idx <- c(bound.idx, which(lower == x))
+                }
+                if(!is.null(lavpartable$upper)) {
+                    bound.idx <- c(bound.idx, which(upper == x))
+                }
+                if(length(bound.idx) > 0L) {
+                    non.zero <- non.zero[- which(non.zero %in% bound.idx) ]
+                }
+
+                # this has many implications ... so should be careful to
+                # avoid false alarm
+                if(length(non.zero) > 0L) {
+                    converged <- FALSE
+                    warn.txt <- paste("the optimizer (", OPTIMIZER, ") ",
+                       "claimed the model converged,\n",
+"                      but not all elements of the gradient are (near) zero;\n",
+"                      the optimizer may not have found a local solution\n",
+"                      use check.gradient = FALSE to skip this check.",
+                       sep = "")
+                }
+            }
+        } else {
+            dx <- numeric(0L)
+        }
+    } else {
+       dx <- numeric(0L)
+       warn.txt <- "the optimizer warns that a solution has NOT been found!"
+    }
+
     # transform back
     # 3.
     #if(lavoptions$optim.var.transform == "sqrt" &&
@@ -686,13 +895,15 @@ lav_model_estimate <- function(lavmodel       = NULL,
     x <- x / parscale
 
     attr(x, "converged")  <- converged
+    attr(x, "warn.txt")   <- warn.txt
     attr(x, "iterations") <- iterations
     attr(x, "control")    <- control
     attr(x, "fx")         <- fx
+    attr(x, "dx")         <- dx
     attr(x, "parscale")   <- parscale
     if(!is.null(optim.out$con.jac)) attr(x, "con.jac")    <- optim.out$con.jac
     if(!is.null(optim.out$lambda))  attr(x, "con.lambda") <- optim.out$lambda
-    if(lavoptions$partrace) {
+    if(lavoptions$optim.partrace) {
         attr(x, "partrace") <- PENV$PARTRACE
     }
 
