@@ -25,6 +25,7 @@ lav_partable_flat <- function(FLAT = NULL,
                               group.equal      = NULL,
                               group.w.free     = FALSE,
                               ngroups          = 1L,
+                              nthresholds      = NULL,
                               ov.names.x.block = NULL) {
 
     categorical <- FALSE
@@ -49,7 +50,6 @@ lav_partable_flat <- function(FLAT = NULL,
     #lvov.names.y <- c(ov.names.y, lv.names.y)
     lvov.names.y <- c(lv.names.y, ov.names.y)
 
-
     # get 'ordered' variables, either from FLAT or varTable
     ov.names.ord1 <- lav_partable_vnames(FLAT, type="ov.ord")
     # check if we have "|" for exogenous variables
@@ -60,8 +60,10 @@ lav_partable_flat <- function(FLAT = NULL,
         }
     }
 
+    # check data
     if(!is.null(varTable)) {
-        ov.names.ord2 <- as.character(varTable$name[ varTable$type == "ordered" ])
+        ov.names.ord2 <-
+            as.character(varTable$name[ varTable$type == "ordered" ])
         # remove fixed.x variables
         idx <- which(ov.names.ord2 %in% ov.names.x)
         if(length(idx) > 0L) {
@@ -74,26 +76,52 @@ lav_partable_flat <- function(FLAT = NULL,
             ov.names.ord2 <- ov.names.ord2[-idx]
         }
     } else {
-        ov.names.ord2 <- character(0)
+        ov.names.ord2 <- character(0L)
     }
-    #### FIXME!!!!! ORDER!
-    ov.names.ord <- unique(c(ov.names.ord1, ov.names.ord2))
 
-    # if we have the "|" in the model syntax, check the number of thresholds
-    if(!is.null(varTable) && length(ov.names.ord1) > 0L) {
-        for(o in ov.names.ord1) {
-            nth <- varTable$nlev[ varTable$name == o ] - 1L
-            nth.in.partable <- sum(FLAT$op == "|" & FLAT$lhs == o)
-            if(nth != nth.in.partable) {
-                stop("lavaan ERROR: expected ", max(0,nth),
-                     " threshold(s) for variable ",
-                     sQuote(o), "; syntax contains ", nth.in.partable, "\n")
+    # check nthresholds, if it is a named vector
+    ov.names.ord3 <- character(0L)
+    if(!is.null(nthresholds)) {
+        if(!is.null(varTable)) {
+            stop("lavaan ERROR: the varTable and nthresholds arguments should not be used together.")
+        }
+        if(!is.numeric(nthresholds)) {
+            stop("lavaan ERROR: nthresholds should be a named vector of integers.")
+        }
+        nth.names <- names(nthresholds)
+        if(!is.null(nth.names)) {
+            ov.names.ord3 <- nth.names
+        } else {
+            # if nthresholds is just a number, all is good; otherwise it
+            # should be a names vector
+            if(length(nthresholds) > 1L) {
+                warning("lavaan ERROR: nthresholds must be a named vector of integers.")
             }
+            # just a single number -> assume ALL y variables are ordered
+            ov.names.ord3 <- ov.names.nox
         }
     }
 
-    if(length(ov.names.ord) > 0L)
+    # final ov.names.ord
+    tmp <- unique(c(ov.names.ord1, ov.names.ord2, ov.names.ord3))
+    ov.names.ord <- ov.names[ ov.names %in% tmp ]
+
+    # if we have the "|" in the model syntax, check the number of thresholds
+    #if(!is.null(varTable) && length(ov.names.ord1) > 0L) {
+    #    for(o in ov.names.ord1) {
+    #        nth <- varTable$nlev[ varTable$name == o ] - 1L
+    #        nth.in.partable <- sum(FLAT$op == "|" & FLAT$lhs == o)
+    #        if(nth != nth.in.partable) {
+    #            stop("lavaan ERROR: expected ", max(0,nth),
+    #                 " threshold(s) for variable ",
+    #                 sQuote(o), "; syntax contains ", nth.in.partable, "\n")
+    #        }
+    #    }
+    #}
+
+    if(length(ov.names.ord) > 0L) {
         categorical <- TRUE
+    }
 
     # std.lv = TRUE, group.equal includes "loadings"
     #if(ngroups > 1L && std.lv && "loadings" %in% group.equal) {
@@ -132,13 +160,28 @@ lav_partable_flat <- function(FLAT = NULL,
     # 1. THRESHOLDS (based on varTable)
     #    NOTE: - new in 0.5-18: ALWAYS include threshold parameters in partable,
     #            but only free them if auto.th = TRUE
-    #          - only ov.names.ord2, because ov.names.ord1 are already in USER
-    #            and we only need to add 'default' parameters here
+    #          - [only ov.names.ord2, because ov.names.ord1 are already in USER
+    #            and we only need to add 'default' parameters here]
+    #            (not any longer: we create them for ALL ordered var (0.6-12)
     nth <- 0L
     #if(auto.th && length(ov.names.ord2) > 0L) {
-    if(length(ov.names.ord2) > 0L) {
-        for(o in ov.names.ord2) {
-            nth  <- varTable$nlev[ varTable$name == o ] - 1L
+    #if(length(ov.names.ord2) > 0L) {
+    if(length(ov.names.ord) > 0L) {
+        #for(o in ov.names.ord2) {
+        for(o in ov.names.ord) {
+            if(!is.null(varTable)) {
+                nth  <- varTable$nlev[ varTable$name == o ] - 1L
+            } else if(!is.null(nthresholds)) {
+                if(length(nthresholds) == 1L && is.null(nth.names)) {
+                    nth <- nthresholds
+                } else {
+                    # we can assume nthresholds is a named vector
+                    nth <- unname(nthresholds[o])
+                    if(is.na(nth)) {
+                        stop("lavaan ERROR: ordered variable ", o, " not found in the named vector nthresholds.")
+                    }
+                }
+            }
             if(nth < 1L) next
             lhs <- c(lhs, rep(o, nth))
             rhs <- c(rhs, paste("t", seq_len(nth), sep=""))
@@ -176,9 +219,30 @@ lav_partable_flat <- function(FLAT = NULL,
 
     # d) exogenous x covariates: VARIANCES + COVARIANCES
     if((nx <- length(ov.names.x)) > 0L) {
-        idx <- lower.tri(matrix(0, nx, nx), diag=TRUE)
-        lhs <- c(lhs, rep(ov.names.x,  each=nx)[idx]) # fill upper.tri
-        rhs <- c(rhs, rep(ov.names.x, times=nx)[idx])
+        if(conditional.x) {
+            # new in 0.6-12: we make a distinction between ov.names.x and
+            # ov.names.x.block: we treat them 'separately' (with no covariances
+            # among them)
+            # but we add 'regressions' instead (see below)
+            ov.names.x1 <- ov.names.x[!ov.names.x %in% ov.names.x.block]
+            ov.names.x2 <- ov.names.x.block
+            nx1 <- length(ov.names.x1) # splitted x
+            nx2 <- length(ov.names.x2) # regular  x
+            if(nx1 > 0L) {
+                idx <- lower.tri(matrix(0, nx1, nx1), diag=TRUE)
+                lhs <- c(lhs, rep(ov.names.x1,  each=nx1)[idx]) # fill upper.tri
+                rhs <- c(rhs, rep(ov.names.x1, times=nx1)[idx])
+            }
+            if(nx2 > 0L) {
+                idx <- lower.tri(matrix(0, nx2, nx2), diag=TRUE)
+                lhs <- c(lhs, rep(ov.names.x2,  each=nx2)[idx]) # fill upper.tri
+                rhs <- c(rhs, rep(ov.names.x2, times=nx2)[idx])
+            }
+        } else {
+            idx <- lower.tri(matrix(0, nx, nx), diag=TRUE)
+            lhs <- c(lhs, rep(ov.names.x,  each=nx)[idx]) # fill upper.tri
+            rhs <- c(rhs, rep(ov.names.x, times=nx)[idx])
+        }
     }
 
     # e) efa latent variables COVARIANCES:
@@ -227,6 +291,24 @@ lav_partable_flat <- function(FLAT = NULL,
         op  <- c(op,  rep("~1", length(int.lhs)))
     }
 
+    # 4. REGRESSIONS
+    if(conditional.x) {
+        # new in 0.6-12: we make a distinction between ov.names.x and
+        # ov.names.x.block: we treat them 'separately' (with no covariances
+        # among them)
+        # but we add 'regressions' instead!
+        ov.names.x1 <- ov.names.x[!ov.names.x %in% ov.names.x.block]
+        ov.names.x2 <- ov.names.x.block
+        nx1 <- length(ov.names.x1) # splitted x
+        nx2 <- length(ov.names.x2) # regular  x
+        if(nx1 > 0L && nx2 > 0L) {
+            # add regressions for splitted-x ~ regular-x
+            lhs <- c(lhs, rep(ov.names.x1, times = nx2))
+             op <- c( op, rep("~", nx2 * nx1))
+            rhs <- c(rhs, rep(ov.names.x2, each = nx1))
+        }
+    }
+
     # free group weights
     if(group.w.free) {
         lhs <- c(lhs, "group")
@@ -456,40 +538,66 @@ lav_partable_flat <- function(FLAT = NULL,
     # 5. handle exogenous `x' covariates
     # usually, ov.names.x.block == ov.names.x
     # except if multilevel, where 'splitted' ov.x are treated as endogenous
-    if(length(ov.names.x.block) > 0) {
+
+    # 5a conditional.x = FALSE
+    if(!conditional.x && fixed.x && length(ov.names.x.block) > 0) {
 
         # 1. variances/covariances
         exo.var.idx  <- which(op == "~~" &
                           rhs %in% ov.names.x.block &
                           lhs %in% ov.names.x.block &
                           user == 0L)
-        if(fixed.x) {
-            ustart[exo.var.idx] <- as.numeric(NA) # should be overriden later!
-              free[exo.var.idx] <- 0L
-               exo[exo.var.idx] <- 1L
-        } #else if(conditional.x) {
-          #     exo[exo.var.idx] <- 1L
-        #}
+        ustart[exo.var.idx] <- as.numeric(NA) # should be overriden later!
+          free[exo.var.idx] <- 0L
+           exo[exo.var.idx] <- 1L
 
         # 2. intercepts
         exo.int.idx  <- which(op == "~1" &
                               lhs %in% ov.names.x.block &
                               user == 0L)
+        ustart[exo.int.idx] <- as.numeric(NA) # should be overriden later!
+          free[exo.int.idx] <- 0L
+           exo[exo.int.idx] <- 1L
+    }
+
+    # 5a-bis. conditional.x = TRUE
+    if(conditional.x && length(ov.names.x) > 0L) {
+        # 1. variances/covariances
+        exo.var.idx  <- which(op == "~~" &
+                          rhs %in% ov.names.x &
+                          lhs %in% ov.names.x &
+                          user == 0L)
+        if(fixed.x) {
+            ustart[exo.var.idx] <- as.numeric(NA) # should be overriden later!
+              free[exo.var.idx] <- 0L
+        }
+        exo[exo.var.idx] <- 1L
+
+        # 2. intercepts
+        exo.int.idx  <- which(op == "~1" &
+                              lhs %in% ov.names.x &
+                              user == 0L)
         if(fixed.x) {
             ustart[exo.int.idx] <- as.numeric(NA) # should be overriden later!
               free[exo.int.idx] <- 0L
-               exo[exo.int.idx] <- 1L
-        } #else if(conditional.x) {
-          #     exo[exo.int.idx] <- 1L
-        #}
+        }
+        exo[exo.int.idx] <- 1L
 
         # 3. regressions ov + lv
         exo.reg.idx <- which(op %in% c("~", "<~") &
                              lhs %in% c(lv.names, ov.names.nox) &
-                             rhs %in% ov.names.x.block)
-        if(conditional.x) {
-            exo[exo.reg.idx] <- 1L
+                             rhs %in% ov.names.x)
+        exo[exo.reg.idx] <- 1L
+
+        # 3b regression splitted.x ~ regular.x
+        exo.reg2.idx <- which(op %in% c("~", "<~") &
+                              lhs %in% ov.names.x &
+                              rhs %in% ov.names.x)
+        if(fixed.x) {
+            ustart[exo.reg2.idx] <- as.numeric(NA) # should be overriden later!
+              free[exo.reg2.idx] <- 0L
         }
+        exo[exo.reg2.idx] <- 1L
     }
 
     # 5b. residual variances of ordinal variables?
