@@ -54,11 +54,20 @@ lavaan <- function(# user-specified model: can be syntax, parameter Table, ...
     start.time0 <- start.time <- proc.time()[3]; timing <- list()
 
     # 0a. store call
-    mc  <- match.call(expand.dots = TRUE)
+    SYSCALL <- sys.call() # to get main arguments without partial matching
+    mc  <- match.call(expand.dots = TRUE) # sys.call to avoid partial matching
 
     # handle dotdotdot
     dotdotdot <- list(...)
 
+    # catch partial matching of 'cl' (expanded to cluster)
+    if(!is.null(SYSCALL[["cl"]]) && is.null(SYSCALL[["cluster"]]) &&
+       !is.null(mc[["cluster"]])) {
+        mc[["cl"]] <- mc[["cluster"]]
+        mc[["cluster"]] <- cluster <- NULL
+        dotdotdot$cl <- SYSCALL[["cl"]]
+    }
+
     # check data
     if(!is.null(data)) {
         if(inherits(data, "data.frame")) {
@@ -98,7 +107,8 @@ lavaan <- function(# user-specified model: can be syntax, parameter Table, ...
                 dotdotdot$control$init_nelder_mead
         }
     }
-
+    timing$init <- (proc.time()[3] - start.time)
+    start.time <- proc.time()[3]
 
     ######################
     #### 1. ov.names  ####
@@ -286,6 +296,17 @@ lavaan <- function(# user-specified model: can be syntax, parameter Table, ...
         }
     }
 
+    # sanity check: we do not support latent interaction yet (using the :)
+    lv.int.idx <- which(grepl(":", LV.names))
+    if(length(lv.int.idx) > 0L) {
+        txt <- c("Interaction terms involving latent variables (",
+                 LV.names[lv.int.idx[1]], ") are not supported.",
+                 " You may consider creating product indicators to define ",
+                 "the latent interaction term. See the indProd() function ",
+                 "in the semTools package.")
+        stop(lav_txt2message(txt, header = "lavaan ERROR:"))
+    }
+
     # handle ov.names.l
     if(any(FLAT$op == ":" & tolower(FLAT$lhs) == "level")) {
 
@@ -397,6 +418,9 @@ lavaan <- function(# user-specified model: can be syntax, parameter Table, ...
     # in the model syntax
     ordered <- unique(c(ordered, lavNames(FLAT, "ov.ord")))
 
+    timing$ov.names <- (proc.time()[3] - start.time)
+    start.time <- proc.time()[3]
+
 
     #######################
     #### 2. lavoptions ####
@@ -404,6 +428,20 @@ lavaan <- function(# user-specified model: can be syntax, parameter Table, ...
     if(!is.null(slotOptions)) {
         lavoptions <- slotOptions
 
+        # backwards compatibility
+        if(!is.null(lavoptions$categorical)) {
+            lavoptions$.categorical <- lavoptions$categorical
+            lavoptions$categorical <- NULL
+        }
+        if(!is.null(lavoptions$clustered)) {
+            lavoptions$.clustered <- lavoptions$clustered
+            lavoptions$clustered <- NULL
+        }
+        if(!is.null(lavoptions$multilevel)) {
+            lavoptions$.multilevel <- lavoptions$multilevel
+            lavoptions$multilevel <- NULL
+        }
+
         # but what if other 'options' are given anyway (eg 'start = ')?
         # give a warning!
         if(length(dotdotdot) > 0L) {
@@ -440,42 +478,47 @@ lavaan <- function(# user-specified model: can be syntax, parameter Table, ...
         }
 
         # categorical mode?
-        opt$categorical <- FALSE
+        opt$.categorical <- FALSE
         if(any(FLAT$op == "|")) {
-            opt$categorical <- TRUE
+            opt$.categorical <- TRUE
         } else if(!is.null(data) && length(ordered) > 0L) {
-            opt$categorical <- TRUE
+            opt$.categorical <- TRUE
         } else if(!is.null(sample.th)) {
-            opt$categorical <- TRUE
+            opt$.categorical <- TRUE
         } else if(is.data.frame(data)) {
             # first check if we can find ov.names.y in Data
             OV.names.y <- unique(unlist(ov.names.y))
+            # remove possible interaction terms involving an y term
+            int.idx <- which(grepl(":", OV.names.y))
+            if(length(int.idx) > 0L) {
+                OV.names.y <- OV.names.y[-int.idx]
+            }
             idx.missing <- which(!(OV.names.y %in% names(data)))
             if(length(idx.missing)) {
                 stop("lavaan ERROR: missing observed variables in dataset: ",
                     paste(OV.names.y[idx.missing], collapse=" "))
             }
             if(any(sapply(data[, OV.names.y], inherits, "ordered")) ) {
-                opt$categorical <- TRUE
+                opt$.categorical <- TRUE
             }
         }
 
         # clustered?
         if(length(cluster) > 0L) {
-            opt$clustered <- TRUE
+            opt$.clustered <- TRUE
         } else {
-            opt$clustered <- FALSE
+            opt$.clustered <- FALSE
         }
 
         # multilevel?
         if(length(ov.names.l) > 0L && length(ov.names.l[[1]]) > 1L) {
-            opt$multilevel <- TRUE
+            opt$.multilevel <- TRUE
         } else {
-            opt$multilevel <- FALSE
+            opt$.multilevel <- FALSE
         }
 
         # sampling weights? force MLR
-        if(!is.null(sampling.weights) && !opt$categorical &&
+        if(!is.null(sampling.weights) && !opt$.categorical &&
            opt$estimator %in% c("default", "ML")) {
             opt$estimator <- "MLR"
         }
@@ -672,6 +715,7 @@ lavaan <- function(# user-specified model: can be syntax, parameter Table, ...
                       fixed.x          = lavoptions$fixed.x,
                       std.lv           = lavoptions$std.lv,
                       effect.coding    = lavoptions$effect.coding,
+                      ceq.simple       = lavoptions$ceq.simple,
                       parameterization = lavoptions$parameterization,
                       auto.fix.first   = lavoptions$auto.fix.first,
                       auto.fix.single  = lavoptions$auto.fix.single,
@@ -711,9 +755,11 @@ lavaan <- function(# user-specified model: can be syntax, parameter Table, ...
     # at this point, we should check if the partable is complete
     # or not; this is especially relevant if the lavaan() function
     # was used, but the user has forgotten some variances/intercepts...
-    junk <- lav_partable_check(lavpartable,
-                               categorical = lavoptions$categorical,
-                               warn = TRUE)
+    if(is.null(slotParTable)) {
+        junk <- lav_partable_check(lavpartable,
+                                   categorical = lavoptions$.categorical,
+                                   warn = TRUE)
+    }
 
     # for EM only (for now), force fixed-to-zero (residual) variances
     # to be slightly larger than zero
@@ -726,6 +772,9 @@ lavaan <- function(# user-specified model: can be syntax, parameter Table, ...
             lavpartable$ustart[zero.var.idx] <- lavoptions$em.zerovar.offset
         }
     }
+    timing$ParTable <- (proc.time()[3] - start.time)
+    start.time <- proc.time()[3]
+
 
     #################################
     #### 4b. parameter attributes ###
@@ -737,7 +786,7 @@ lavaan <- function(# user-specified model: can be syntax, parameter Table, ...
     if(lavoptions$verbose) {
         cat(" done.\n")
     }
-    timing$ParTable <- (proc.time()[3] - start.time)
+    timing$lavpta <- (proc.time()[3] - start.time)
     start.time <- proc.time()[3]
 
 
@@ -763,6 +812,11 @@ lavaan <- function(# user-specified model: can be syntax, parameter Table, ...
         if(lavoptions$verbose) {
             cat("lavsamplestats ...")
         }
+        # check if we have sample.mean and meanstructure = TRUE
+        if(lavoptions$meanstructure && is.null(sample.mean)) {
+            txt <- "sample.mean= argument is missing, but model contains mean/intercept parameters."
+            warning(lav_txt2message(txt))
+        }
         lavsamplestats <- lav_samplestats_from_moments(
                            sample.cov    = sample.cov,
                            sample.mean   = sample.mean,
@@ -770,13 +824,9 @@ lavaan <- function(# user-specified model: can be syntax, parameter Table, ...
                            sample.nobs   = sample.nobs,
                            ov.names      = ov.names,
                            ov.names.x    = ov.names.x,
-                           estimator     = lavoptions$estimator,
-                           mimic         = lavoptions$mimic,
-                           group.w.free  = lavoptions$group.w.free,
                            WLS.V         = WLS.V,
                            NACOV         = NACOV,
-                           ridge         = lavoptions$ridge,
-                           rescale       = lavoptions$sample.cov.rescale)
+                           lavoptions    = lavoptions)
         if(lavoptions$verbose) {
             cat(" done.\n")
         }
@@ -814,6 +864,9 @@ lavaan <- function(# user-specified model: can be syntax, parameter Table, ...
                                            lavsamplestats = lavsamplestats,
                                            lavpta         = lavpta,
                                            lavoptions     = lavoptions)
+                if(lavoptions$debug) {
+                    print(out)
+                }
                 h1.implied      <- out$implied
                 h1.loglik       <- out$logl$loglik
                 h1.loglik.group <- out$logl$loglik.group
@@ -846,7 +899,9 @@ lavaan <- function(# user-specified model: can be syntax, parameter Table, ...
     #############################
 
     # automatic bounds (new in 0.6-6)
-    if(!is.null(lavoptions$optim.bounds)) {
+    if(!is.null(lavoptions$optim.bounds) ||
+       length(lavoptions$optim.bounds$lower) > 0L ||
+       length(lavoptions$optim.bounds$upper) > 0L) {
         if(lavoptions$verbose) {
             cat("lavpartable bounds ...")
         }
@@ -857,6 +912,8 @@ lavaan <- function(# user-specified model: can be syntax, parameter Table, ...
             cat(" done.\n")
         }
     }
+    timing$bounds <- (proc.time()[3] - start.time)
+    start.time <- proc.time()[3]
 
 
 
@@ -912,6 +969,7 @@ lavaan <- function(# user-specified model: can be syntax, parameter Table, ...
             START <- lav_start(start.method   = lavoptions$start,
                                lavpartable    = lavpartable,
                                lavsamplestats = lavsamplestats,
+                               lavh1          = lavh1,
                                model.type     = lavoptions$model.type,
                                reflect      = FALSE,
                                #order.lv.by  = lavoptions$rotation.args$order.lv.by,
@@ -1290,6 +1348,12 @@ lavaan <- function(# user-specified model: can be syntax, parameter Table, ...
 
         # Quasi-Newton
         } else {
+
+            # for backwards compatibility (<0.6)
+            if(is.null(lavoptions$optim.attempts)) {
+                lavoptions$optim.attempts <- 1L
+            }
+
             # try 1
             if(lavoptions$verbose) {
                 cat("attempt 1 -- default options\n")
@@ -1431,6 +1495,7 @@ lavaan <- function(# user-specified model: can be syntax, parameter Table, ...
     lavoptim$converged  <- attr(x, "converged")
     lavoptim$warn.txt   <- attr(x, "warn.txt")
     lavoptim$parscale   <- attr(x, "parscale")
+    lavoptim$partrace   <- attr(x, "partrace")
     fx.copy <- fx <- attr(x, "fx"); attributes(fx) <- NULL
     lavoptim$fx         <- fx
     lavoptim$fx.group   <- attr(fx.copy, "fx.group")
@@ -1619,6 +1684,7 @@ lavaan <- function(# user-specified model: can be syntax, parameter Table, ...
     #######################
     lavfit <- lav_model_fit(lavpartable = lavpartable,
                             lavmodel    = lavmodel,
+                            lavimplied  = lavimplied,
                             x           = x,
                             VCOV        = VCOV,
                             TEST        = TEST)
@@ -1837,6 +1903,8 @@ lavaan <- function(# user-specified model: can be syntax, parameter Table, ...
             }
         } # vcov
     } # efa
+    timing$rotation <- (proc.time()[3] - start.time)
+    start.time <- proc.time()[3]
 
 
     ####################
@@ -1923,8 +1991,16 @@ cfa <- sem <- function(# user-specified model: can be syntax, parameter Table
                        # options (dotdotdot)
                        ...) {
 
+    SYSCALL <- sys.call() # no partial matching
     mc <- match.call(expand.dots = TRUE)
 
+    # catch partial matching of 'cl' (expanded to cluster)
+    if(!is.null(SYSCALL[["cl"]]) && is.null(SYSCALL[["cluster"]]) &&
+       !is.null(mc[["cluster"]])) {
+        mc[["cl"]] <- mc[["cluster"]]
+        mc[["cluster"]] <- NULL
+    }
+
     # set model.type
     mc$model.type      = as.character( mc[[1L]] )
     if(length(mc$model.type) == 3L) {
@@ -1990,8 +2066,16 @@ growth <- function(# user-specified model: can be syntax, parameter Table
                    # options (dotdotdot)
                    ...) {
 
+    SYSCALL <- sys.call() # no partial matching
     mc <- match.call(expand.dots = TRUE)
 
+    # catch partial matching of 'cl' (expanded to cluster)
+    if(!is.null(SYSCALL[["cl"]]) && is.null(SYSCALL[["cluster"]]) &&
+       !is.null(mc[["cluster"]])) {
+        mc[["cl"]] <- mc[["cluster"]]
+        mc[["cluster"]] <- NULL
+    }
+
     # set model.type
     mc$model.type      = as.character( mc[[1L]] )
     if(length(mc$model.type) == 3L) {
